/*
 * RoboDroid Wars
 * Copyright (C) 2006 Dawid Ciezarkiewicz <dawid.ciezarkiewicz@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA 
 */

/**
 * License: GNU GPL
 *
 * Author: Dawid Ciężarkiewicz <dawid.ciezarkiewicz@gmail.com>
 */
module map;

import player;
import battle;
import error;

import cell.cell;
import cell.wall;
import cell.spawnpoint;
import cell.ground;
import cell.medkit;
import cell.movable;
import droid;

import std.stream;
import std.ctype;
import std.random;

enum Direction {
	UP,
	DOWN,
	UP_LEFT,
	UP_RIGHT,
	DOWN_LEFT,
	DOWN_RIGHT
}

struct Position {
	int x;
	int y;
}

/**
 * Map (consisting of Cells) on which simulation
 * "really" happens - objects are interacting etc.
 */
class Map {
	private Cell cells[][];
	private Battle battle;
	private uint xSize, ySize;
	private SpawnPoint spawnPoints[];

	public this(Battle parent, char[] name) {
		this.battle = parent;

		File input = new File("data/maps/" ~ name, FileMode.In);
		input.readf("%d %d", &xSize, &ySize);

		cells.length = xSize;
		foreach (inout Cell row[]; cells) {
			row.length = ySize;
		}

		scope (failure) {
			deleteAllCells();
		}

		for (int y = 0; y < ySize; y++) {
			for (int x = 0; x < xSize;) {

				char ch = input.getc();
				if (ch == char.init) {
					throw new InvalidArgument("`" ~ name ~ "' - unexpected"
							" EOF");
				}

				if (!.isgraph(ch))
					continue;

				switch (ch) {
					case '#':
						cells[x][y] = new Wall(this, x, y);
						break;

					case ',':
						SpawnPoint sp = new SpawnPoint(this, x, y);
						cells[x][y] = sp;
						spawnPoints ~= sp;
						break;

					case '.':
						cells[x][y] = new Ground(this, x, y);
						break;

					case '+':
						cells[x][y] = new Medkit(this, x, y);

					default:
						throw new InvalidArgument("`" ~ name ~ "' map has "
								~ "unsupported char: `" ~ ch ~ "'");
				}
				x++;
			}
		}

		if (spawnPoints.length < 2) {
			throw new InvalidArgument("map `" ~ name
					~ "' has not enough spawn points");
		}

		for (int i = 0 ; i < spawnPoints.length; ++i) {
			shuffleSpawnPoints();
		}
	}

	/**
	 * Delete all cells that this map consists of.
	 */
	protected void deleteAllCells() {
		foreach (inout Cell row[]; cells) {
			foreach (inout Cell cell; row) {
				cell = null;
			}
		}
		cells = null;
	}

	/**
	 * Shuffle spawn points table a bit.
	 */
	protected void shuffleSpawnPoints() {
		debug assert (spawnPoints.length > 2);
		int i = rand() % spawnPoints.length;
		int j = rand() % spawnPoints.length;

		SpawnPoint tmp = spawnPoints[i];
		spawnPoints[i] = spawnPoints[j];
		spawnPoints[j] = tmp;
	}

	/**
	 * Destructor.
	 */
	public ~this() {
		deleteAllCells();
	}

	/**
	 * Spaw droid for player.
	 *
	 * Params:
	 * player = player for which droid will be spawned
	 *
	 * Returns:
	 * Spawned Droid.
	 *
	 * Throws:
	 * InvalidArgument on failure.
	 */
	public Droid spawnDroid(Player player) {
		foreach (SpawnPoint sp; spawnPoints) {
			if (sp.trySpawnDroid(player)) {
				shuffleSpawnPoints();
				return new Droid(player, this,
						sp.getX(), sp.getY());
			}
		}

		throw new InvalidArgument("couldn't spawn `" ~ player.getName()
				~ "' -  no free spawn points available");
	}

	/**
	 * Run single tick.
	 */
	public void runTick() {
	}

	/**
	 * Get cell at (x, y) position.
	 */
	public Cell getCell(uint x, uint y) {
		debug assert (x < xSize);
		debug assert (y < ySize);

		return cells[x][y];
	}
}

