/*
 * RoboDroid Wars
 * Copyright (C) 2006 Dawid Ciezarkiewicz <dawid.ciezarkiewicz@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA 
 */

/**
 * License: GNU GPL
 *
 * Author: Dawid Ciężarkiewicz <dawid.ciezarkiewicz@gmail.com>
 */
module options;

private {
	import std.conv;
	import std.string;
	import std.stdio;
	import error;
	import std.stdio;
}

/**
 * Command line options parsing.
 */
class Options {
	private char[][] battles;
	private char[][] players;
	private uint speed;

	/**
	 * Init with command line arguments.
	 */
	public this(char[][] args) {

		speed = 5;
		enum Arg {
			UNINIT,
			PLAYER,
			BATTLE,
			SPEED
		};

		Arg next_arg = Arg.UNINIT;
		foreach (char[] arg; args[1 .. $]) {
			switch (next_arg) {
				case Arg.UNINIT:
					switch (arg) {
						case "-p":
							next_arg = Arg.PLAYER;
						break;

						case "-b":
							next_arg = Arg.BATTLE;
						break;

						case "-s":
							next_arg = Arg.SPEED;
						break;

						case "-h":
							case "--help":
							printUsage();
						throw new SimulationEndRequest();

						default:
						throw new InvalidArgument(arg);
					}
					break;

				case Arg.PLAYER:
					players ~= arg;
					next_arg = Arg.UNINIT;
					break;

				case Arg.BATTLE:
					battles ~= arg;
					next_arg = Arg.UNINIT;
					break;

				case Arg.SPEED:
					speed = toInt(arg);
					if (speed < 1 || speed > 9) {
						throw new InvalidArgument("speed = "
								~ .toString(speed));
					}
					next_arg = Arg.UNINIT;
					break;
				default:
					assert(0);
			}
		}

		if (players.length < 2) {
			throw new InvalidArgument("not enough players");
		}

		if (battles.length == 0) {
			throw new InvalidArgument("not enough battles");
		}
	}

	/**
	 * Get FX driver name.
	 */
	public char[] getFXDriverName() {
		return "sdl";
	}

	/**
	 * Get screen width.
	 */
	public uint getVidX() {
		return 800;
	}

	/**
	 * Get screen height.
	 */
	public uint getVidY() {
		return 640;
	}

	/**
	 * Get registered battle names that simulation
	 * should start.
	 */
	public char[][] getBattleNames() {
		return battles;
	}

	/**
	 * Get players list.
	 */
	public char[][] getPlayerNames() {
		return players;
	}

	/**
	 * Get game speed.
	 */
	public uint getSimulationSpeed() {
		return speed;
	}

	/**
	 * Print command line usage.
	 */
	public void printUsage() {
		writefln("params: ");
		writefln(" -b <battle>");
		writefln(" -p <player>");
		writefln(" -s <speed>");
	}

}

